from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import operator, string
from reportlab.platypus import *
#from reportlab import rl_config
from reportlab.lib.styles import PropertySet, getSampleStyleSheet, ParagraphStyle
from reportlab.lib import colors
from reportlab.lib.units import inch
from reportlab.platypus.paragraph import Paragraph
#from reportlab.lib.utils import fp_str
#from reportlab.pdfbase import pdfmetrics
from reportlab.platypus.flowables import PageBreak
import os
import unittest

class TableTestCase(unittest.TestCase):

    def getDataBlock(self):
        "Helper - data for our spanned table"
        return [
            # two rows are for headers
            ['Region','Product','Period',None,None,None,'Total'],
            [None,None,'Q1','Q2','Q3','Q4',None],

            # now for data
            ['North','Spam',100,110,120,130,460],
            ['North','Eggs',101,111,121,131,464],
            ['North','Guinness',102,112,122,132,468],

            ['South','Spam',100,110,120,130,460],
            ['South','Eggs',101,111,121,131,464],
            ['South','Guinness',102,112,122,132,468],
            ]

    def test_document(self):

        rowheights = (24, 16, 16, 16, 16)
        rowheights2 = (24, 16, 16, 16, 30)
        colwidths = (50, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32)
        GRID_STYLE = TableStyle(
            [('GRID', (0,0), (-1,-1), 0.25, colors.black),
             ('ALIGN', (1,1), (-1,-1), 'RIGHT')]
            )

        styleSheet = getSampleStyleSheet()
        styNormal = styleSheet['Normal']
        styNormal.spaceBefore = 6
        styNormal.spaceAfter = 6

        lst = []
        lst.append(Paragraph("""Oversized cells""", styleSheet['Heading1']))

        lst.append(Paragraph("""Cells can end up being larger than a page.
        In that case, we need to split the cell. splitByRow and splitInRow
        controls that. By default splitByRow is 1 and splitInRow is 0.
        It splits between two rows. """,
        styNormal))

        ministy = TableStyle([
            ('GRID', (0,0), (-1,-1), 1.0, colors.black),
            ('VALIGN', (0,1), (1,1), 'BOTTOM'),
            ('VALIGN', (1,1), (2,1), 'MIDDLE'),
            ('VALIGN', (2,1), (3,1), 'TOP'),
            ('VALIGN', (3,1), (4,1), 'BOTTOM'),
            ('VALIGN', (4,1), (5,1), 'MIDDLE'),
            ('VALIGN', (5,1), (6,1), 'TOP'),
            ])
        cell1 = [Paragraph(
            """This is a very tall cell to make a tall row.""",
            styNormal)]
        cell2 = [Paragraph("A cell with two flowables.", styNormal),
            Paragraph("And valign= MIDDLE.", styNormal)]
        cell3 = [Paragraph("Paragraph with valign=TOP", styNormal)]

        tableData = [
            ['Row 1', 'Two rows:\nSo there', 'is a', 'place', 'to split', 'the table'],
            [cell1, cell2, cell3, 'valign=BOTTOM', 'valign=MIDDLE', 'valign=TOP']
        ]
        colWidths = (50, 75, 70, 90, 90, 70)

        # This is the table with splitByRow, which splits between row 1 & 2:
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=1,
                  splitInRow=0)
        parts = t.split(451, 60)
        lst.append(parts[0])
        lst.append(Spacer(0,6))
        lst.append(parts[1])

        lst.append(Paragraph("""Here is the same table, with splitByRow=0 and
        splitInRow=1. It splits inside a row.""",
        styNormal))

        # This is the table with splitInRow, which splits in row 2:
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=1)

        parts = t.split(451, 60)
        lst.append(parts[0])
        lst.append(Spacer(0,6))
        lst.append(parts[1])

        lst.append(Paragraph("""Here is the same table, with splitByRow=1 and
        splitInRow=1. It splits between the rows, if possible.""",
        styNormal))

        # This is the table with both splits, which splits in row 2:
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=1,
                  splitInRow=1)

        parts = t.split(451, 60)
        lst.append(parts[0])
        lst.append(Spacer(0,6))
        lst.append(parts[1])

        lst.append(Paragraph("""But if we constrict the space to less than the first row,
        it splits that row.""",
        styNormal))

        # This is the table with both splits and no space, which splits in row 1:
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=1,
                  splitInRow=1)

        parts = t.split(451, 15)
        lst.append(parts[0])
        lst.append(Spacer(0,6))
        lst.append(parts[1])

        # Split it at a point in row 2, where the split fails
        lst.append(Paragraph("""When splitByRow is 0 and splitInRow is 1, we should
        still allow fallback to splitting between rows""",
        styNormal))

        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=1)
        parts = t.split(451, 50)
        lst.append(parts[0])
        lst.append(Spacer(0,6))
        lst.append(parts[1])

        #create extra page keep in xlist
        xlst=[].append
        xlst(PageBreak())
        xlst(Paragraph("""Modifying the inrow split linebelow and lineabove""", styleSheet['Heading1']))
        xlst(Paragraph("""Table, with splitByRow=0 and splitInRow=1. It splits inside a row without
                       split row line below and line above (the normal case).""", styNormal))

        # This is the table with splitInRow, which splits in row 2
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=1)

        parts = t.split(451, 60)
        xlst(parts[0])
        xlst(Spacer(0,6))
        xlst(parts[1])
        xlst(Paragraph("""Same Table: splits inside a row without
                       With thicker blue LINEBELOW & red LINEABOVE""", styNormal))
        newsty = TableStyle([
                ('LINEBELOW', (0,'inrowsplitstart'), (-1,'inrowsplitstart'), 1, colors.blue,'butt'),
                ('LINEABOVE', (0,'inrowsplitend'), (-1,'inrowsplitend'), 1, colors.red,'butt'),
            ],parent=ministy)
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=newsty,
                  splitByRow=0,
                  splitInRow=1)

        parts = t.split(451, 60)
        xlst(parts[0])
        xlst(Spacer(0,6))
        xlst(parts[1])

        xlst(Paragraph("""Table and repeatrow, with splitByRow=0 and splitInRow=1. It splits inside a row without
                       split row line below and line above (the normal case).""", styNormal))

        xTableData = [['Spanned repeat row',"","","","",""]]+tableData
        # This is the table with splitInRow, which splits in row 2
        t = Table(xTableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=1,
                  repeatRows=1)

        parts = t.split(451, 75)
        xlst(parts[0])
        xlst(Spacer(0,6))
        xlst(parts[1])
        xlst(Paragraph("""Same Table: splits inside a row without
                       With thicker blue LINEBELOW & red LINEABOVE""", styNormal))
        newsty = TableStyle([
                ('LINEBELOW', (0,'inrowsplitstart'), (-1,'inrowsplitstart'), 1, colors.blue,'butt'),
                ('LINEABOVE', (0,'inrowsplitend'), (-1,'inrowsplitend'), 1, colors.red,'butt'),
            ],parent=ministy)
        t = Table(xTableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=newsty,
                  splitByRow=0,
                  splitInRow=1,
                  repeatRows=1)

        parts = t.split(451, 75)
        xlst(parts[0])
        xlst(Spacer(0,6))
        xlst(parts[1])

        lst.append(PageBreak())
        lst.append(Paragraph("""Long cell with multiple splits, and minimum split size""",
                             styleSheet['Heading2']))

        lst.append(Paragraph("With a height of 80 amd splitInRow=1 (no minimum rest) "
                             "we get a small last split.",
                             styleSheet['Normal']))

        ministy = TableStyle([('GRID', (0,0), (-1,-1), 1.0, colors.black),])
        cell1 = [Paragraph(
            "This is a very very tall cell to make a very very tall row so we can split it "
            "many many times, and also test for minimum splits.""",
            styNormal)]

        tableData = [['Row 1', 'Row 1'],[cell1, 'Cell2']]
        colWidths = (50,50)

        # This is the table with splitByRow, which splits between row 1 & 2:
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=1)

        while True:
            s = t.split(4*inch, 80)
            lst.append(s[0])
            lst.append(Spacer(6,6))
            if len(s) > 1:
                t = s[1]
            else:
                break

        # Now the same, but with a minimum size of 40.
        lst.append(Paragraph("With minimum split size to 40 (splitInRow=40) the last split "
                             "isn't done, and it should instead flow over to the next page.",
                             styleSheet['Normal']))
        t = Table(tableData,
                  colWidths=colWidths,
                  rowHeights=None,
                  style=ministy,
                  splitByRow=0,
                  splitInRow=40)

        while True:
            s = t.split(4*inch, 80)
            if s:
                lst.append(s[0])
                lst.append(Spacer(6,6))
            else:
                lst.append(t)
            if len(s) > 1:
                t = s[1]
            else:
                break

        # It should also not split at less than 40 in the beginning.
        assert not t.split(4*inch, 30)

        lst.append(PageBreak())
        lst.append(Paragraph("""Style handling splitInRow""", styleSheet['Heading2']))

        tableData = [
            ['00\n\naa', '01', '02', '03', '04'],
            ['10', '11\nbb', '12', '13', '14'],
            ['20', '21', '22\ncc', '23', '24'],
            ['30', '31', '32', '33\ndd', '34']
        ]
        styles = [
            ('GRID',(0,0),(-1,-1),0.5,colors.grey),
            ('GRID',(1,1),(-2,-2),1,colors.green),
            ('BOX',(0,0),(1,-1),2,colors.red),
            ('BOX',(0,0),(-1,-1),2,colors.black),
            ('LINEABOVE',(1,2),(-2,2),1,colors.blue),
            ('LINEBEFORE',(2,1),(2,-2),1,colors.pink),
            ('BACKGROUND', (0, 0), (0, 1), colors.pink),
            ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
            ('BACKGROUND', (2, 2), (2, 3), colors.orange),
            ('TEXTCOLOR',(0,-1),(-2,-1),colors.green),
        ]

        t = Table(tableData,style=styles, splitInRow=1, splitByRow=0)
        for split in (36, 60, 100, 130,):
            parts = t.split(4*inch, split)
            lst.append(parts[0])
            lst.append(Spacer(0,6))
            lst.append(parts[1])
            lst.append(Spacer(0,12))

        lst.append(PageBreak())
        lst.append(Paragraph("""Splitfirst/splitlast behavior with split rows and spans""",
                             styleSheet['Heading2']))

        data=  [['A', 'BBBBB', 'C', 'D', 'E'],
                ['00', '01', '02', '03', '04'],
                ['10\n11', ],
                ['20', '21', '22', '23', '24'],
                ['30', '31', '32', '33', '34']]
        sty = [
                ('ALIGN',(0,0),(-1,-1),'CENTER'),
                ('VALIGN',(0,0),(-1,-1),'TOP'),
                ('GRID',(0,0),(-1,-1),1,colors.green),
                ('BOX',(0,0),(-1,-1),2,colors.red),

                #span 'BBBB' across middle 3 cells in top row
                ('SPAN',(1,0),(3,0)),
                #now color the first cell in this range only,
                #i.e. the one we want to have spanned.  Hopefuly
                #the range of 3 will come out khaki.
                ('BACKGROUND',(1,0),(1,0),colors.khaki),

                ('SPAN',(0,2),(-1,2)),

                #span 'AAA'down entire left column
                ('SPAN',(0,0), (0, 1)),
                ('BACKGROUND',(0,0),(0,0),colors.cyan),
                ('TEXTCOLOR', (0,'splitfirst'), (-1,'splitfirst'), colors.cyan),
                ('TEXTCOLOR', (0,'splitlast'), (-1,'splitlast'), colors.red),
                ('BACKGROUND', (0,'splitlast'), (-1,'splitlast'), colors.pink),
                ('LINEBELOW', (0,'splitlast'), (-1,'splitlast'), 1, colors.grey,'butt'),
               ]
        t=Table(data,style=sty, colWidths = [20] * 5, splitInRow=1, splitByRow=0)
        lst.append(t)
        lst.append(Spacer(18,18))

        for split in (40, 60, 75):
            for s in t.split(4*inch, split):
                lst.append(s)
                lst.append(Spacer(0,6))
            lst.append(Spacer(18,12))

        lst.append(PageBreak())
        lst.append(Paragraph("""Style handling with rowspans""", styleSheet['Heading2']))

        data = [
            ['R0C0\nR1C0\nR2C0', 'R0C1', 'R0C2', 'R0C3', 'R0C4\nR1C4'],
            ['', 'R1C1', 'R1C2', 'R1C3', ''],
            ['', 'R2C1', 'R2C2', 'R2C3', 'R2C4'],
            ['R3C0', 'R3C1', 'R3C2', 'R3C3 R3C4\nR4C3 R4C4', ''],
            ['R4C0', 'R4C1', 'R4C2', '', ''],
        ]

        sty = [
                ('GRID',(0,0),(-1,-1),0.5,colors.grey),
                ('GRID',(1,1),(-2,-2),1,colors.green),
                ('BOX',(0,0),(-1,-1),3,colors.black),
                ('BOX',(0,0),(1,-1),2,colors.red),
                ('LINEABOVE',(1,2),(-2,2),1,colors.blue),
                ('LINEBEFORE',(2,1),(2,-2),1,colors.pink),
                ('BACKGROUND', (0, 0), (0, 0), colors.pink),
                ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
                ('BACKGROUND', (2, 3), (2, 4), colors.orange),
                ('TEXTCOLOR',(0,0),(-1,0),colors.brown),
                ('TEXTCOLOR',(1,1),(-2,-1),colors.green),
                ('SPAN',(0,0), (0, 2)),
                ('SPAN',(2,0), (2, 1)),
                ('SPAN',(3,3), (4, 4)),
                ('SPAN',(4,0), (4, 1)),
                ]
        t=Table(data,style=sty, colWidths = [40] * 5, splitInRow=1, splitByRow=0)
        lst.append(t)
        lst.append(Spacer(18,18))

        for split in (30, 45, 60, 75):
            for s in t.split(4*inch, split):
                lst.append(s)
                lst.append(Spacer(0,6))
            lst.append(Spacer(18,18))

        lst.append(PageBreak())
        lst.append(Paragraph("""More Style handling with rowspans""", styleSheet['Heading2']))

        data = [
            ['R0C0', 'R0C1', 'R0C2', 'R0C3', 'R0C4'],
            ['R1C0', 'R1C1', 'R1C2', 'R1C3', 'R1C4'],
            ['R2C0', 'R2C1', 'R2C2', 'R2C3', 'R2C4'],
            ['R3C0', 'R3C1', 'R3C2', 'R3C3', 'R3C4'],
            ['R4C0', 'R4C1', 'R4C2', 'R4C3', 'R4C4'],
        ]

        sty = [
                ('GRID',(0,0),(-1,-1),0.5,colors.grey),
                ('GRID',(1,1),(1,-1),1,colors.green),
                ('BOX',(0,0),(-3,-3),3,colors.black),
                ('BOX',(1,1),(-1,-1),2,colors.red),
                ('LINEABOVE',(1,1),(2,2),1,colors.blue),
                ('LINEBELOW',(2,2),(4,4),1,colors.pink),
                ('BACKGROUND', (0, 0), (0, 0), colors.pink),
                ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
                ('BACKGROUND', (4, 3), (4, 4), colors.orange),
                ('TEXTCOLOR',(0,0),(-1,0),colors.brown),
                ('TEXTCOLOR',(1,1),(-2,-1),colors.green),
                ('SPAN',(0,0), (0, -1)),
                ('SPAN',(2,1), (-2, 3)),
                ]
        t=Table(data,style=sty, colWidths = [40] * 5, splitInRow=1, splitByRow=0)
        lst.append(t)
        lst.append(Spacer(18,18))

        for split in (30, 45, 60, 75):
            for s in t.split(4*inch, split):
                lst.append(s)
                lst.append(Spacer(0,6))
            lst.append(Spacer(18,18))

        lst.append(PageBreak())
        lst.append(Paragraph("""Testing for a couple of height alignment bugs""", styleSheet['Heading2']))

        data = [
            ['R0C0\nR1C0\nR2C0', 'R0C1', 'R0C2', 'R0C3', 'R0C4\nR1C4', 'R0C5'],
            ['', 'R1C1', 'R1C2', 'R1C3', '', 'R1C5\nR2C5\nR3C5'],
            ['', 'R2C1', 'R2C2', 'R2C3', 'R2C4', ''],
            ['R3C0', 'R3C1', 'R3C2', 'R3C3 R3C4\nR4C3 R4C4', '', ''],
            ['R4C0', 'R4C1', 'R4C2', '', '', 'R4C5'],
        ]

        sty = [
                ('GRID',(0,0),(-1,-1),0.5,colors.grey),
                ('GRID',(1,1),(-2,-2),1,colors.green),
                ('BOX',(0,0),(-1,-1),3,colors.black),
                ('BOX',(0,0),(1,-1),2,colors.red),
                ('LINEABOVE',(1,2),(-2,2),1,colors.blue),
                ('LINEBEFORE',(2,1),(2,-2),1,colors.pink),
                ('BACKGROUND', (0, 0), (0, 0), colors.pink),
                ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
                ('BACKGROUND', (2, 3), (2, 4), colors.orange),
                ('BACKGROUND',(5,1),(5,3),colors.greenyellow),
                ('BACKGROUND',(5,4),(5,4),colors.darkviolet),
                ('TEXTCOLOR',(0,0),(-1,0),colors.brown),
                ('TEXTCOLOR',(1,1),(-2,-1),colors.green),
                ('TEXTCOLOR',(5,1),(5,3),colors.magenta),
                ('TEXTCOLOR',(5,4),(5,4),colors.white),
                ('SPAN',(0,0), (0, 2)),
                ('SPAN',(2,0), (2, 1)),
                ('SPAN',(3,3), (4, 4)),
                ('SPAN',(4,0), (4, 1)),
                ('SPAN',(5,1), (5, 3)),
                ]
        t=Table(data,style=sty, colWidths = [40] * 6, splitInRow=1, splitByRow=0)
        lst.append(t)
        lst.append(Spacer(0,6))

        for split in (30, 40, 50, 60):
            for s in t.split(4*inch, split):
                lst.append(s)
                lst.append(Spacer(0,6))
            lst.append(Spacer(18,18))

        lst.extend(xlst.__self__)
        SimpleDocTemplate(outputfile('test_table_inrowsplit.pdf'), showBoundary=1).build(lst)

    def test_list_in_row(self):
        LOREM = """Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."""

        bulleted_data = [
            {'indent': 0, 'para': 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.'},
            {'indent': 0, 'para': 'Lorem Ipsum has been the industry standard dummy text ever since the 1500s.'},
            {'indent': 1, 'para': '1 0 It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged.'},
            {'indent': 1, 'para': '1 1 Contrary to popular belief, Lorem Ipsum is not simply random text. '+LOREM},

            # pad this one out repeatedly until it breaks over the page at level 3
            {'indent': 2, 'para': 'Indent level 2, with enough stuff to continue over the page break...'+LOREM},
            {'indent': 2, 'para': 'This paragraph should be at the third level of indentation. Now let us have too much content for a page... '+LOREM},
            {'indent': 2, 'para': '2 2 '+LOREM}, {'indent': 2, 'para': '2 3 '+LOREM}, {'indent': 2, 'para': '2 4 '+LOREM}, {'indent': 2, 'para': '2 5 '+LOREM},
            {'indent': 2, 'para': '2 6 '+LOREM}, {'indent': 2, 'para': '2 7 '+LOREM}, {'indent': 2, 'para': '2 8 '+LOREM},
            {'indent': 1,'para': 'Back to level one.'},
            {'indent': 1,'para': 'All the Lorem Ipsum generators on the Internet tend to repeat predefined chunks as necessary.'},
            ]

        class LFL(list):
            pass

        def makeUList():
            # This holds the structured list.
            list_stack = []
            # Prep default style for Paragraph
            para_style = getSampleStyleSheet()['Normal']
            # Prep default style for ListFlowable
            style = getSampleStyleSheet()['UnorderedList']
            style.bulletFontSize = 12
            style.bulletType = 'bullet'
            style.leftIndent = 12 * 1.2
            # Walk the list to build the nested structure
            for next_item in bulleted_data:
                # Get the indent value
                indent = next_item['indent']
                # Get the para content
                para = next_item['para']
                # If we have nothing, put empty list at lowest level
                if len(list_stack) == 0:
                    list_stack.append([])
                # If indent value matches length of stack, start new list
                if indent == len(list_stack):
                    list_stack.append([])
                    cur_list_array = list_stack[-1]
                    cur_list_array.append(Paragraph(para, para_style))
                # If indentation is reduced
                elif indent < len(list_stack) - 1:
                    i = len(list_stack) - 1
                    while i > indent:
                        ulist = LFL((list_stack[-1]))
                        list_stack.pop()
                        cur_list_array = list_stack[-1]
                        cur_list_array.append(ulist)
                        i = i - 1
                    cur_list_array = list_stack[-1]
                    cur_list_array.append(Paragraph(para, para_style))
                # If indentation is same
                else:
                    cur_list_array = list_stack[-1]
                    cur_list_array.append(Paragraph(para, para_style))

            # Close remaining lists
            ulist = None
            while len(list_stack) > 0:
                ulist = LFL(list_stack[-1])
                list_stack.pop()
                if len(list_stack) > 0:
                    cur_list_array = list_stack[-1]
                    cur_list_array.append(ulist)

            from reportlab.platypus.flowables import _bulletNames
            BT = list(_bulletNames.keys())
            def findLFL(F,depth=0):
                for i,f in enumerate(F):
                    if isinstance(f,LFL):
                        F[i] = findLFL(F[i],depth+1)
                return ListFlowable(flowables=F,
                              style=style.clone(name=f'gen-{depth}-{i}',start=BT[depth]))
            ulist = findLFL(ulist,depth=0)
            return ulist

        story = []
        styGrid = TableStyle([
                ('GRID', (0,0), (-1,-1), 0.5, colors.green),
                ('TOPPADDING', (0,0), (-1,-1), 3),
                ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                ('LEFTPADDING', (0,0), (-1,-1), 0),
                ('RIGHTPADDING', (0,0), (-1,-1), 0),
                #('VALIGN',(0,0), (-1,-1), "MIDDLE"),
                ])
        tbl = Table([
            [makeUList()],
            ],
            splitInRow = 1,
            style=styGrid)
        story.append(tbl)

        SimpleDocTemplate(outputfile('test_table_listsplit.pdf'), showBoundary=1).build(story)

    def test_small_para_inrowsplit(self):
        '''test bug reported by Christian Zwicknagl via Yoshua Wakeham'''

        data = [([Paragraph(f"({x})" + " This is a small paragraph." * 4) for x in range(38)],)]
        colWidths = 200

        t = Table(
            data,
            colWidths=colWidths,
            splitInRow=1,
        )

        SimpleDocTemplate(outputfile("test_small_para_inrowsplit.pdf"), showBoundary=1).build([t])

def makeSuite():
    return makeSuiteForClasses(TableTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    print(f'saved {outputfile("test_table_inrowsplit.pdf")}')
    print(f' and  {outputfile("test_table_listsplit.pdf")}')
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Ellipse, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Ellipse(200,100,50,50,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(200,40,'a',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, Line, String, PolyLine
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(50,50,300,125,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,50,9.245484,125,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(59.24548,50,57.70457,125,rx=0,ry=0,fillColor=Color(.678431,.847059,.901961,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(116.9501,50,58.661,125,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(175.6111,50,57.70457,125,rx=0,ry=0,fillColor=Color(.678431,.847059,.901961,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(233.3156,50,58.661,125,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(291.9766,50,58.02338,125,rx=0,ry=0,fillColor=Color(.678431,.847059,.901961,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(350,50,0,125,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=None,strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,350,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(59.24548,50,59.24548,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(116.9501,50,116.9501,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(175.6111,50,175.6111,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(233.3156,50,233.3156,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(291.9766,50,291.9766,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(350,50,350,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,59.24548,42)
		v0.add(String(0,-4,'12/97',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,116.9501,42)
		v0.add(String(0,-4,'06/98',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,175.6111,42)
		v0.add(String(0,-4,'12/98',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,233.3156,42)
		v0.add(String(0,-4,'06/99',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,291.9766,42)
		v0.add(String(0,-4,'12/99',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (.707107,-0.707107,.707107,.707107,350,42)
		v0.add(String(0,-4,'06/00',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Line(50,50,50,175,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,45,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,89.55696,45,89.55696,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,129.1139,45,129.1139,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,168.6709,45,168.6709,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,50)
		v0.add(String(-30.83,-4,'  100% ',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,89.55696)
		v0.add(String(-30.83,-4,'  105% ',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,129.1139)
		v0.add(String(-30.83,-4,'  110% ',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,168.6709)
		v0.add(String(-30.83,-4,'  115% ',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(PolyLine(points=[50,50,59.24548,51.34839,69.12859,62.37308,78.05526,67.31007,87.93836,62.92505,97.50266,71.86744,107.3858,73.09971,116.9501,75.7863,126.8332,75.72502,136.7163,93.31039,146.2806,123.768,156.1637,144.8254,165.7279,136.3803,175.6111,136.3803,185.4942,144.8438,194.4208,127.4625,204.3039,131.0277,213.8682,132.4995,223.7513,123.8889,233.3156,115.1438,243.1987,130.7986,253.0818,136.7746,262.6461,148.4378,272.5292,150.7231,282.0935,147.7133,291.9766,147.8328,301.8597,136.4361,311.1052,133.2803,320.9883,155.3134,330.5526,137.3806,340.4357,142.5848,350,157.4622],strokeColor=PCMYKColor(100,65,0,30,spotName='PANTONE 288 CV',alpha=100),strokeWidth=2,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[50,50,59.24548,50,69.12859,56.32911,78.05526,65.82278,87.93836,65.03165,97.50266,73.73418,107.3858,73.73418,116.9501,74.52532,126.8332,74.52532,136.7163,72.1519,146.2806,94.3038,156.1637,115.6646,165.7279,114.0823,175.6111,144.1456,185.4942,153.6392,194.4208,130.6962,204.3039,143.3544,213.8682,147.3101,223.7513,129.9051,233.3156,123.5759,243.1987,138.6076,253.0818,142.5633,262.6461,149.6835,272.5292,154.4304,282.0935,159.9684,291.9766,171.8354,301.8597,150.4747,311.1052,159.9684,320.9883,175,330.5526,146.519,340.4357,149.6835,350,165.5063],strokeColor=PCMYKColor(0,79,91,0,spotName='PANTONE Wm Red CV',alpha=100),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=[3,1],strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Line(100,10,100,90,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(100,10,95,10,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(100,32.85714,95,32.85714,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(100,55.71429,95,55.71429,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(100,78.57143,95,78.57143,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,95,10)
		v0.add(String(-18.33,-4,'10%',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,95,32.85714)
		v0.add(String(-18.33,-4,'15%',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,95,55.71429)
		v0.add(String(-18.33,-4,'20%',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,95,78.57143)
		v0.add(String(-18.33,-4,'25%',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#a class that will convert a pdfpath definition to our format
from reportlab.graphics.shapes import Path, definePath, _PATH_OP_ARG_COUNT, _PATH_OP_NAMES
__all__ = ('PDFPath',)

def _getSegs(L):
    n = len(L)
    i = 0
    ops = dict(m='moveTo',l='lineTo',c='curveTo',h='closePath')
    while i < n:
        for j in i, i+2, i+6:
            op = L[j]
            if op in ops:
                try:
                    opName = ops[op]
                    nargs = _PATH_OP_ARG_COUNT[_PATH_OP_NAMES.index(opName)]
                    yield tuple([opName]+[float(L[i+k]) for k in range(nargs)])
                    i = j+1
                    break
                except:
                    raise ValueError('Error converting PDFPath at %s' % ' '.join(L[i:i+6]))
        else:
            raise ValueError('Error converting PDFPath at %s' % ' '.join(L[i:i+6]))

def pdfpath(pdf='',**kwds):
    if pdf:
        pdf = pdf.strip()
    if pdf:
        p = definePath(pathSegs=list(_getSegs(pdf.split())),**kwds)
    else:
        p = Path(**kwds)
    return p



#!/bin/env python
# coding=utf8
#Copyright ReportLab Europe Ltd. 2025
#see license.txt for license details
__version__='3.3.0'
__doc__="""Exercising basic Canvas operations and libraries involved in Arabic/Hebrew


This is really to build up oour own understanding at the moment.
"""
from reportlab.lib.testutils import (setOutDir,makeSuiteForClasses, outputfile,
                                    printLocation, rlSkipUnless, haveDejaVu, )
setOutDir(__name__)
import unittest
from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfgen.textobject import rtlSupport, log2vis, bidiWordList, BidiStr, bidiShapedText, bidiText
from reportlab.platypus.paraparser import _greekConvert
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont, uharfbuzz, freshTTFont, ShapedStr, shapeFragWord, shapeStr
from reportlab.platypus.paragraph import Paragraph
from reportlab.lib.styles import getSampleStyleSheet, TA_RIGHT
from reportlab.lib.abag import ABag

class RtlTestCase(unittest.TestCase):
    "Simplest test that makes PDF"

    @rlSkipUnless(rtlSupport and haveDejaVu() and uharfbuzz,'miss RTL and/or DejaVu/uharfbuzz')
    def test_arabic(self):
        c = Canvas(outputfile('test_arabic.pdf'))

        pdfmetrics.registerFont(TTFont("DejaVuSans", "DejaVuSans.ttf"))

        c.setFont('Helvetica-Bold', 18)
        c.drawString(100,800, 'Tests relating to RTL, Hebrew and Arabic')
        c.setFont('Helvetica', 12)


        if rtlSupport:  # essential for these tests
            import rlbidi
            msg = f"Using rlbidi version {rlbidi.__version__}"
            c.drawString(100,750, msg)

        else:
            msg = "rlbidi not imported, exiting"
            c.drawString(100, 750, msg)
            c.save()
            return

        if uharfbuzz:
            msg = f"Using uharfbuzz version {uharfbuzz.__version__}"
        else:
            msg = "uharfbuzz not imported, this may not matter"
        c.drawString(100, 730, msg)


        phrase = "أنت مجنون"
        unichars = [hex(ord(ch)) for ch in phrase]
        c.drawString(100, 715, "Our test phrase is taken from the Rowan Atkinson Barclaycard advert in 2011.")
        c.drawString(100, 700, "https://www.youtube.com/watch?v=WEDCGCwY8zg")

        c.drawString(100, 685, '          "\'ant majnun", meaning "you\'re crazy"')
        c.drawString(100, 670, "The characters in this in Helvetica are not defined:" + str(repr(list(phrase))))

        c.setFont("DejaVuSans", 12)
        c.drawString(100, 655, "The characters in this in DejaVuSans are" + str(repr(list(phrase))))
        c.drawString(100, 640, "That's 3 characters, a space and 5 characters in the input.")

        c.drawString(100, 625, "Byte values (unicode) = " + repr(unichars))

        c.drawString(100, 610, "Raw left to right display (VERY wrong)" + phrase)

        logicalchars = log2vis(phrase)
        c.drawString(100, 595, "Output of log2vis = " + logicalchars)
        unichars = [hex(ord(ch)) for ch in logicalchars]
        c.drawString(100, 580, "Char values in log2vis = " + repr(unichars))

        c.drawString(100, 565, "This appears correct (although DejaVu is less pretty than Arial)")

        width_raw = pdfmetrics.stringWidth(phrase, "DejaVuSans", 12)
        width_log = pdfmetrics.stringWidth(logicalchars, "DejaVuSans", 12)

        c.drawString(100, 550, f"Raw string width = {width_raw}, Modified string width = {width_log}")
        c.drawString(100, 535, f"We will always overestimate the width, ")
        c.drawString(100, 520, f"so will mess it up when centered or right-aligned")
        c.save()

    @rlSkipUnless(rtlSupport and haveDejaVu(),'miss RTL and/or DejaVu')
    def test_bidiWordList(self):
        c = Canvas(outputfile('test_arabic_bidiWordList.pdf'))
        fontName = 'DejaVuSans'
        fontSize = 12
        leading = 14.4
        pdfmetrics.registerFont(freshTTFont("DejaVuSans", "DejaVuSans.ttf"))
        sss = getSampleStyleSheet()
        W,H = c._pagesize
        styN = sss.Normal.clone('Normal',fontName='DejaVuSans',bulletFontName='DejaVuSans',
                          fontSize=12,leading=leading,allowOrphans=True)
        styNRTL = styN.clone('NormalRTL',wordWrap='RTL', alignment=TA_RIGHT)#, backColor=(0,0.2,0))
        c.setFont(fontName, fontSize, leading=leading)
        class ExampleP:
            def __init__(self, x, y, p, aW, aH, sC):
                self.w, self.h = p.wrap(aW,aH)
                self.params = x, y, p, aW, aH, sC
            def draw(self):
                x, y, p, aW, aH, sC = self.params
                y -= self.h
                c.saveState()
                c.setStrokeColor(sC)
                c.rect(x,y-0.2*styNRTL.fontSize,aW,self.h)
                c.restoreState()
                p.drawOn(c, x=x, y=y)

        def example(eng, arabic, dw=0):
            fW = W - 72 #ie width - 2*36
            x = 36
            y = H - 36
            w = pdfmetrics.stringWidth(eng,fontName,fontSize)
            if w>fW:
                c.saveState()
                c.setFontSize(fontSize*fW/w)
                c.drawString(x,y,eng)
                c.restoreState()
            else:
                c.drawString(x,y,eng)
            p = ExampleP(x, y, Paragraph(arabic,style=styNRTL), fW,H-72,(1,0,0))
            p.draw()

            rW = W/3
            x = W - 36 - rW
            y -= p.h + leading
            p = ExampleP(x, y, Paragraph(arabic,style=styNRTL), rW, H-72,(1,0,0))
            p.draw()
            y -= p.h+2*leading
            c.drawString(x,y,'Splitting the arabic')
            P = Paragraph(arabic,style=styNRTL)
            P1, P2 = P.split(rW, leading+1)
            p = ExampleP(x, y, P1,rW,leading+.1,(0,0.8,0))
            p.draw()
            y -= leading
            p = ExampleP(x+dw, y, P2,rW-dw,leading+.1,(0,0,0.8))
            p.draw()
            c.showPage()
        example('Al Riyadh is an Arab football team with 123 expensive players',
                '\u0627\u0644\u0631\u064a\u0627\u0636 \u0647\u0648 \u0641\u0631'
                '\u064a\u0642 \u0643\u0631\u0629 \u0642\u062f\u0645 \u0639\u0631'
                '\u0628\u064a \u064a\u0636\u0645 123 \u0644\u0627\u0639\u0628\u064b'
                '\u0627 \u0628\u0627\u0647\u0638 \u0627\u0644\u062b\u0645\u0646',
                )

        example('Al Riyadh is an Arab football team with 123 expensive players including a'
                ' Golden Boot winner and other foreign internationals.',
                '\u0627\u0644\u0631\u064a\u0627\u0636 \u0647\u0648 \u0641\u0631\u064a\u0642'
                ' \u0643\u0631\u0629 \u0642\u062f\u0645 \u0639\u0631\u0628\u064a \u064a\u0636'
                '\u0645 123 \u0644\u0627\u0639\u0628\u0627\u064b \u0628\u0627\u0647\u0638'
                ' \u0627\u0644\u062b\u0645\u0646 \u0628\u0645\u0627 \u0641\u064a \u0630'
                '\u0644\u0643 \u0627\u0644\u0641\u0627\u0626\u0632 \u0628\u0627\u0644'
                '\u062d\u0630\u0627\u0621 \u0627\u0644\u0630\u0647\u0628\u064a \u0648'
                '\u0639\u062f\u062f \u0645\u0646 \u0627\u0644\u0644\u0627\u0639\u0628'
                '\u064a\u0646 \u0627\u0644\u0623\u062c\u0627\u0646\u0628 \u0627\u0644'
                '\u062f\u0648\u0644\u064a\u064a\u0646.',
                dw = 15
                )
        c.save()

    @rlSkipUnless(rtlSupport,'miss RTL support')
    def test_bidiStrWrap(self):
        from reportlab.platypus.paragraph import _SHYStr, _SplitWordH, _SplitWordEnd
        from reportlab.pdfgen.textobject import BidiStr, bidiStrWrap, _bidiKS

        for i,klass in enumerate((_SHYStr, _SplitWordH, _SplitWordEnd)):
            s = 'abc\xaddef' if klass==_SHYStr else 'ABCDEF'
            orig = BidiStr('dontcare',bidiV=79+i)
            ks = klass(s)
            instw = bidiStrWrap(ks,orig)
            self.assertTrue(hasattr(instw,'__bidiV__'))
            self.assertTrue(instw.__bidiV__==orig.__bidiV__)
            self.assertTrue(instw==ks)
            self.assertTrue(bidiStrWrap(instw,orig) is instw)
            if hasattr(ks,'__dict__'):
                for k in ks.__dict__:
                    self.assertEqual(getattr(ks,k),getattr(instw,k),
                                    f'ks.{k}={getattr(ks,k)} not equal to instw.{k}={getattr(instw,k)}') 

    words0 = 'الرياض هو فريق كرة قدم عربي يضم 123 لاعبًا باهظ الثمن'.split()
    words0 = '\u0627\u0644\u0631\u064a\u0627\u0636 \u0647\u0648 \u0641\u0631\u064a\u0642 \u0643\u0631\u0629 \u0642\u062f\u0645 \u0639\u0631\u0628\u064a \u064a\u0636\u0645 123 \u0644\u0627\u0639\u0628\u064b\u0627 \u0628\u0627\u0647\u0638 \u0627\u0644\u062b\u0645\u0646'.split()

    @rlSkipUnless(rtlSupport,'miss RTL support')
    def test_making_words(self):
        bwl = bidiWordList(self.words0)
        self.assertEqual(bwl,['\ufebd\ufe8e\ufef3\ufeae\ufedf\ufe8d', '\ufeee\ufeeb', '\ufed6\ufef3\ufeae\ufed3', '\ufe93\ufeae\ufedb', '\ufee1\ufeaa\ufed7', '\ufef2\ufe91\ufeae\ufecb', '\ufee2\ufec0\ufef3', '123', '\ufe8e\ufe92\u064b\ufecb\ufefb', '\ufec6\ufeeb\ufe8e\ufe91', '\ufee6\ufee4\ufe9c\ufedf\ufe8d'])
        rt = set([_.__class__.__name__ for _ in bwl])
        xt = set(['BidiStr'])
        self.assertEqual(rt, xt, f"bidiWordList returned wrong types {rt} not expected {xt}")
        xx = [10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
        rx = [getattr(_,'__bidiV__','?') for _ in bwl]
        self.assertEqual(xx, rx, f"bidiWordList returned wrong indeces {rx} not expected {xx}")
        wx = bidiWordList(self.words0,wx=True)
        self.assertEqual(xx, wx, f"bidiWordList(wx=True) returned wrong indeces {wx} not expected {xx}")

    @rlSkipUnless(rtlSupport and haveDejaVu(),'miss RTL and/or DejaVu')
    def test_bidiShapedText(self):
        c = Canvas(outputfile('test_arabic_bidiShapedText.pdf'))
        fontName = 'DejaVuSans'
        fontSize = 12
        leading = 1.2*fontSize
        pdfmetrics.registerFont(freshTTFont("DejaVuSans", "DejaVuSans.ttf", shapable=True))
        c.setFont(fontName, fontSize, leading)
        x = 36
        y = c._pagesize[1] - 36
        dx = max((pdfmetrics.stringWidth(f'{_}: ',fontName,fontSize) for _ in ('raw rtl','raw rtl shaped')))
        def example(x,y,words):
            raw = ' '.join(words)
            x1 = x+dx
            for direction in ('','rtl'):
                for shaping in (False,True):
                    label = f"raw{' rtl' if direction else ''}{' shaped' if shaping else ''}"
                    c.drawString(x,y,label)
                    c.drawString(x1,y,raw, direction=direction, shaping=shaping)
                    y -= leading
            return y

        y = example(x,y,self.words0)

        if False:
            #Ben's example
            y -= leading
            city = "Jizan"
            ar_month = "ﻣﺎﺭﺱ"
            ar_date = f"13-16 {ar_month} 2025"
            start_time = "START TIME: 23:00"
            benWithParen = f'({city}) - {ar_date} - {start_time}'
            benNoParen = f'{city} - {ar_date} - {start_time}'
            y = example(x,y,benWithParen.split())
            y -= leading
            y = example(x,y,benNoParen.split())
            y -= leading
            google = "جيزان - 13-16 مارس 2025 - وقت البدء: 23:00"
            y = example(x,y,google.split())
            y -= leading
    
        c.save()

def makeSuite():
    return makeSuiteForClasses(RtlTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://hg.reportlab.com/hg-public/reportlab/log/tip/tools/docco/examples.py
testannotations="""
def annotations(canvas):
    from reportlab.lib.units import inch
    canvas.drawString(inch, 2.5*inch,
       "setAuthor, setTitle, setSubject have no visible effect")
    canvas.drawString(inch, inch, "But if you are viewing this document dynamically")
    canvas.drawString(inch, 0.5*inch, "please look at File/Document Info")
    canvas.setAuthor("the ReportLab Team")
    canvas.setTitle("ReportLab PDF Generation User Guide")
    canvas.setSubject("How to Generate PDF files using the ReportLab modules")
"""

# magic function making module

test1 = """
def f(a,b):
    print("it worked", a, b)
    return a+b
"""

test2 = """
def g(n):
    if n==0: return 1
    else: return n*g(n-1)
"""

testhello = """
def hello(c):
    from reportlab.lib.units import inch
    # move the origin up and to the left
    c.translate(inch,inch)
    # define a large font
    c.setFont("Helvetica", 14)
    # choose some colors
    c.setStrokeColorRGB(0.2,0.5,0.3)
    c.setFillColorRGB(1,0,1)
    # draw some lines
    c.line(0,0,0,1.7*inch)
    c.line(0,0,1*inch,0)
    # draw a rectangle
    c.rect(0.2*inch,0.2*inch,1*inch,1.5*inch, fill=1)
    # make text go straight up
    c.rotate(90)
    # change color
    c.setFillColorRGB(0,0,0.77)
    # say hello (note after rotate the y coord needs to be negative!)
    c.drawString(0.3*inch, -inch, "Hello World")
"""

testcoords = """
def coords(canvas):
    from reportlab.lib.units import inch
    from reportlab.lib.colors import pink, black, red, blue, green
    c = canvas
    c.setStrokeColor(pink)
    c.grid([inch, 2*inch, 3*inch, 4*inch], [0.5*inch, inch, 1.5*inch, 2*inch, 2.5*inch])
    c.setStrokeColor(black)
    c.setFont("Times-Roman", 20)
    c.drawString(0,0, "(0,0) the Origin")
    c.drawString(2.5*inch, inch, "(2.5,1) in inches")
    c.drawString(4*inch, 2.5*inch, "(4, 2.5)")
    c.setFillColor(red)
    c.rect(0,2*inch,0.2*inch,0.3*inch, fill=1)
    c.setFillColor(green)
    c.circle(4.5*inch, 0.4*inch, 0.2*inch, fill=1)
"""

testtranslate = """
def translate(canvas):
    from reportlab.lib.units import cm
    canvas.translate(2.3*cm, 0.3*cm)
    coords(canvas)
    """

testscale = """
def scale(canvas):
    canvas.scale(0.75, 0.5)
    coords(canvas)
"""

testscaletranslate = """
def scaletranslate(canvas):
    from reportlab.lib.units import inch
    canvas.setFont("Courier-BoldOblique", 12)
    # save the state
    canvas.saveState()
    # scale then translate
    canvas.scale(0.3, 0.5)
    canvas.translate(2.4*inch, 1.5*inch)
    canvas.drawString(0, 2.7*inch, "Scale then translate")
    coords(canvas)
    # forget the scale and translate...
    canvas.restoreState()
    # translate then scale
    canvas.translate(2.4*inch, 1.5*inch)
    canvas.scale(0.3, 0.5)
    canvas.drawString(0, 2.7*inch, "Translate then scale")
    coords(canvas)
"""

testmirror = """
def mirror(canvas):
    from reportlab.lib.units import inch
    canvas.translate(5.5*inch, 0)
    canvas.scale(-1.0, 1.0)
    coords(canvas)
"""

testRGBcolors = """
def colorsRGB(canvas):
    from reportlab.lib import colors
    from reportlab.lib.units import inch
    black = colors.black
    y = x = 0; dy=inch*3/4.0; dx=inch*5.5/5; w=h=dy/2; rdx=(dx-w)/2
    rdy=h/5.0; texty=h+2*rdy
    canvas.setFont("Helvetica",10)
    for [namedcolor, name] in (
           [colors.lavenderblush, "lavenderblush"],
           [colors.lawngreen, "lawngreen"],
           [colors.lemonchiffon, "lemonchiffon"],
           [colors.lightblue, "lightblue"],
           [colors.lightcoral, "lightcoral"]):
        canvas.setFillColor(namedcolor)
        canvas.rect(x+rdx, y+rdy, w, h, fill=1)
        canvas.setFillColor(black)
        canvas.drawCentredString(x+dx/2, y+texty, name)
        x = x+dx
    y = y + dy; x = 0
    for rgb in [(1,0,0), (0,1,0), (0,0,1), (0.5,0.3,0.1), (0.4,0.5,0.3)]:
        r,g,b = rgb
        canvas.setFillColorRGB(r,g,b)
        canvas.rect(x+rdx, y+rdy, w, h, fill=1)
        canvas.setFillColor(black)
        canvas.drawCentredString(x+dx/2, y+texty, "r%s g%s b%s"%rgb)
        x = x+dx
    y = y + dy; x = 0
    for gray in (0.0, 0.25, 0.50, 0.75, 1.0):
        canvas.setFillGray(gray)
        canvas.rect(x+rdx, y+rdy, w, h, fill=1)
        canvas.setFillColor(black)
        canvas.drawCentredString(x+dx/2, y+texty, "gray: %s"%gray)
        x = x+dx
"""

testCMYKcolors = """
def colorsCMYK(canvas):
    from reportlab.lib.colors import CMYKColor, PCMYKColor
    from reportlab.lib.units import inch
    # creates a black CMYK ; CMYKColor use real values
    black = CMYKColor(0,0,0,1)
    # creates a cyan CMYK ; PCMYKColor use integer values
    cyan = PCMYKColor(100,0,0,0)
    y = x = 0; dy=inch*3/4.0; dx=inch*5.5/5; w=h=dy/2; rdx=(dx-w)/2
    rdy=h/5.0; texty=h+2*rdy
    canvas.setFont("Helvetica",10)
    y = y + dy; x = 0
    for cmyk in [(1,0,0,0), (0,1,0,0), (0,0,1,0), (0,0,0,1), (0,0,0,0)]:
        c,m,y1,k = cmyk
        canvas.setFillColorCMYK(c,m,y1,k)
        canvas.rect(x+rdx, y+rdy, w, h, fill=1)
        canvas.setFillColor(black)
        canvas.drawCentredString(x+dx/2, y+texty, "c%s m%s y%s k%s"%cmyk)
        x = x+dx
"""

testspumoni = """
def spumoni(canvas):
    from reportlab.lib.units import inch
    from reportlab.lib.colors import pink, green, brown, white
    x = 0; dx = 0.4*inch
    for i in range(4):
        for color in (pink, green, brown):
            canvas.setFillColor(color)
            canvas.rect(x,0,dx,3*inch,stroke=0,fill=1)
            x = x+dx
    canvas.setFillColor(white)
    canvas.setStrokeColor(white)
    canvas.setFont("Helvetica-Bold", 85)
    canvas.drawCentredString(2.75*inch, 1.3*inch, "SPUMONI")
"""

testspumoni2 = """
def spumoni2(canvas):
    from reportlab.lib.units import inch
    from reportlab.lib.colors import pink, green, brown, white, black
    # draw the previous drawing
    spumoni(canvas)
    # now put an ice cream cone on top of it:
    # first draw a triangle (ice cream cone)
    p = canvas.beginPath()
    xcenter = 2.75*inch
    radius = 0.45*inch
    p.moveTo(xcenter-radius, 1.5*inch)
    p.lineTo(xcenter+radius, 1.5*inch)
    p.lineTo(xcenter, 0)
    canvas.setFillColor(brown)
    canvas.setStrokeColor(black)
    canvas.drawPath(p, fill=1)
    # draw some circles (scoops)
    y = 1.5*inch
    for color in (pink, green, brown):
        canvas.setFillColor(color)
        canvas.circle(xcenter, y, radius, fill=1)
        y = y+radius
"""

testalpha = """
def alpha(canvas):
    from reportlab.graphics.shapes import Rect
    from reportlab.lib.colors import Color, black, blue, red
    red50transparent = Color( 100, 0, 0, alpha=0.5)
    c = canvas 
    c.setFillColor(black)
    c.setFont('Helvetica', 10)
    c.drawString(25,180, 'solid')
    c.setFillColor(blue)
    c.rect(25,25,100,100, fill=True, stroke=False)
    c.setFillColor(red)
    c.rect(100,75,100,100, fill=True, stroke=False)
    c.setFillColor(black)
    c.drawString(225,180, 'transparent')
    c.setFillColor(blue)
    c.rect(225,25,100,100, fill=True, stroke=False)
    c.setFillColor(red50transparent)
    c.rect(300,75,100,100, fill=True, stroke=False)
"""

testoverprint = """
def overPrint(canvas):
    from reportlab.graphics.shapes import Rect
    from reportlab.lib.colors import PCMYKColor, PCMYKColorSep
    black = PCMYKColorSep(0, 0, 0, 100, spotName='black',density=100)
    blue  = PCMYKColorSep(91.0,  43.0,  0.0, 0.0, spotName='PANTONE 285 CV',density=100)
    red   = PCMYKColorSep( 0.0, 100.0, 91.0, 0.0, spotName='PANTONE 485 CV',density=100)
    c = canvas  
    c.setFillColor(black)
    c.setFont('Helvetica', 10)
    c.drawString(25,180, 'overprint')
    c.setFillOverprint(True)
    c.setFillColor(blue)
    c.rect(25,25,100,100, fill=True, stroke=False)
    c.setFillColor(red)
    c.rect(100,75,100,100, fill=True, stroke=False)
    c.setFillColor(black)
    c.drawString(225,180, 'knockout')
    c.setFillOverprint(False)
    c.setFillColor(blue)
    c.rect(225,25,100,100, fill=True, stroke=False)
    c.setFillColor(red)
    c.rect(300,75,100,100, fill=True, stroke=False)
"""

testbezier = """
def bezier(canvas):
    from reportlab.lib.colors import yellow, green, red, black
    from reportlab.lib.units import inch
    i = inch
    d = i/4
    # define the bezier curve control points
    x1,y1, x2,y2, x3,y3, x4,y4 = d,1.5*i, 1.5*i,d, 3*i,d, 5.5*i-d,3*i-d
    # draw a figure enclosing the control points
    canvas.setFillColor(yellow)
    p = canvas.beginPath()
    p.moveTo(x1,y1)
    for (x,y) in [(x2,y2), (x3,y3), (x4,y4)]:
        p.lineTo(x,y)
    canvas.drawPath(p, fill=1, stroke=0)
    # draw the tangent lines
    canvas.setLineWidth(inch*0.1)
    canvas.setStrokeColor(green)
    canvas.line(x1,y1,x2,y2)
    canvas.setStrokeColor(red)
    canvas.line(x3,y3,x4,y4)
    # finally draw the curve
    canvas.setStrokeColor(black)
    canvas.bezier(x1,y1, x2,y2, x3,y3, x4,y4)
"""

testbezier2 = """
def bezier2(canvas):
    from reportlab.lib.colors import yellow, green, red, black
    from reportlab.lib.units import inch
    # make a sequence of control points
    xd,yd = 5.5*inch/2, 3*inch/2
    xc,yc = xd,yd
    dxdy = [(0,0.33), (0.33,0.33), (0.75,1), (0.875,0.875),
            (0.875,0.875), (1,0.75), (0.33,0.33), (0.33,0)]
    pointlist = []
    for xoffset in (1,-1):
        yoffset = xoffset
        for (dx,dy) in dxdy:
            px = xc + xd*xoffset*dx
            py = yc + yd*yoffset*dy
            pointlist.append((px,py))
        yoffset = -xoffset
        for (dy,dx) in dxdy:
            px = xc + xd*xoffset*dx
            py = yc + yd*yoffset*dy
            pointlist.append((px,py))
    # draw tangent lines and curves
    canvas.setLineWidth(inch*0.1)
    while pointlist:
        [(x1,y1),(x2,y2),(x3,y3),(x4,y4)] = pointlist[:4]
        del pointlist[:4]
        canvas.setLineWidth(inch*0.1)
        canvas.setStrokeColor(green)
        canvas.line(x1,y1,x2,y2)
        canvas.setStrokeColor(red)
        canvas.line(x3,y3,x4,y4)
        # finally draw the curve
        canvas.setStrokeColor(black)
        canvas.bezier(x1,y1, x2,y2, x3,y3, x4,y4)
"""

testpencil = """
def pencil(canvas, text="No.2"):
    from reportlab.lib.colors import yellow, red, black,white
    from reportlab.lib.units import inch
    u = inch/10.0
    canvas.setStrokeColor(black)
    canvas.setLineWidth(4)
    # draw erasor
    canvas.setFillColor(red)
    canvas.circle(30*u, 5*u, 5*u, stroke=1, fill=1)
    # draw all else but the tip (mainly rectangles with different fills)
    canvas.setFillColor(yellow)
    canvas.rect(10*u,0,20*u,10*u, stroke=1, fill=1)
    canvas.setFillColor(black)
    canvas.rect(23*u,0,8*u,10*u,fill=1)
    canvas.roundRect(14*u, 3.5*u, 8*u, 3*u, 1.5*u, stroke=1, fill=1)
    canvas.setFillColor(white)
    canvas.rect(25*u,u,1.2*u,8*u, fill=1,stroke=0)
    canvas.rect(27.5*u,u,1.2*u,8*u, fill=1, stroke=0)
    canvas.setFont("Times-Roman", 3*u)
    canvas.drawCentredString(18*u, 4*u, text)
    # now draw the tip
    penciltip(canvas,debug=0)
    # draw broken lines across the body.
    canvas.setDash([10,5,16,10],0)
    canvas.line(11*u,2.5*u,22*u,2.5*u)
    canvas.line(22*u,7.5*u,12*u,7.5*u)
    """

testpenciltip = """
def penciltip(canvas, debug=1):
    from reportlab.lib.colors import tan, black, green
    from reportlab.lib.units import inch
    u = inch/10.0
    canvas.setLineWidth(4)
    if debug:
        canvas.scale(2.8,2.8) # make it big
        canvas.setLineWidth(1) # small lines
    canvas.setStrokeColor(black)
    canvas.setFillColor(tan)
    p = canvas.beginPath()
    p.moveTo(10*u,0)
    p.lineTo(0,5*u)
    p.lineTo(10*u,10*u)
    p.curveTo(11.5*u,10*u, 11.5*u,7.5*u, 10*u,7.5*u)
    p.curveTo(12*u,7.5*u, 11*u,2.5*u, 9.7*u,2.5*u)
    p.curveTo(10.5*u,2.5*u, 11*u,0, 10*u,0)
    canvas.drawPath(p, stroke=1, fill=1)
    canvas.setFillColor(black)
    p = canvas.beginPath()
    p.moveTo(0,5*u)
    p.lineTo(4*u,3*u)
    p.lineTo(5*u,4.5*u)
    p.lineTo(3*u,6.5*u)
    canvas.drawPath(p, stroke=1, fill=1)
    if debug:
        canvas.setStrokeColor(green) # put in a frame of reference
        canvas.grid([0,5*u,10*u,15*u], [0,5*u,10*u])
"""

testnoteannotation = """
from reportlab.platypus.flowables import Flowable
class NoteAnnotation(Flowable):
    '''put a pencil in the margin.'''
    def wrap(self, *args):
        return (1,10) # I take up very little space! (?)
    def draw(self):
        canvas = self.canv
        canvas.translate(-10,-10)
        canvas.rotate(180)
        canvas.scale(0.2,0.2)
        pencil(canvas, text="NOTE")
"""

testhandannotation = """
from reportlab.platypus.flowables import Flowable
from reportlab.lib.colors import tan, green
class HandAnnotation(Flowable):
    '''A hand flowable.'''
    def __init__(self, xoffset=0, size=None, fillcolor=tan, strokecolor=green):
        from reportlab.lib.units import inch
        if size is None: size=4*inch
        self.fillcolor, self.strokecolor = fillcolor, strokecolor
        self.xoffset = xoffset
        self.size = size
        # normal size is 4 inches
        self.scale = size/(4.0*inch)
    def wrap(self, *args):
        return (self.xoffset, self.size)
    def draw(self):
        canvas = self.canv
        canvas.setLineWidth(6)
        canvas.setFillColor(self.fillcolor)
        canvas.setStrokeColor(self.strokecolor)
        canvas.translate(self.xoffset+self.size,0)
        canvas.rotate(90)
        canvas.scale(self.scale, self.scale)
        hand(canvas, debug=0, fill=1)
"""

lyrics = '''\
To be, or not to be, that is the question:
Whether 'tis nobler in the mind to suffer
The slings and arrows of outrageous fortune,
Or to take arms against a sea of troubles
And by opposing end them. To die—to sleep,
No more; and by a sleep to say we end
The heart-ache and the thousand natural shocks
That flesh is heir to: 'tis a consummation
Devoutly to be wish'd. To die, to sleep;
To sleep, perchance to dream—ay, there's the rub:'''

lyrics = lyrics.split("\n")
testtextsize = """
def textsize(canvas):
    from reportlab.lib.units import inch
    from reportlab.lib.colors import magenta, red
    canvas.setFont("Times-Roman", 20)
    canvas.setFillColor(red)
    canvas.drawCentredString(2.75*inch, 2.5*inch, "Font size examples")
    canvas.setFillColor(magenta)
    size = 7
    y = 2.3*inch
    x = 1.3*inch
    for line in lyrics:
        canvas.setFont("Helvetica", size)
        canvas.drawRightString(x,y,"%s points: " % size)
        canvas.drawString(x,y, line)
        y = y-size*1.2
        size = size+1.5
"""

teststar = """
def star(canvas, title="Title Here", aka="Comment here.",
         xcenter=None, ycenter=None, nvertices=5):
    from math import pi
    from reportlab.lib.units import inch
    radius=inch/3.0
    if xcenter is None: xcenter=2.75*inch
    if ycenter is None: ycenter=1.5*inch
    canvas.drawCentredString(xcenter, ycenter+1.3*radius, title)
    canvas.drawCentredString(xcenter, ycenter-1.4*radius, aka)
    p = canvas.beginPath()
    p.moveTo(xcenter,ycenter+radius)
    from math import pi, cos, sin
    angle = (2*pi)*2/5.0
    startangle = pi/2.0
    for vertex in range(nvertices-1):
        nextangle = angle*(vertex+1)+startangle
        x = xcenter + radius*cos(nextangle)
        y = ycenter + radius*sin(nextangle)
        p.lineTo(x,y)
    if nvertices==5:
       p.close()
    canvas.drawPath(p)
"""

testjoins = """
def joins(canvas):
    from reportlab.lib.units import inch
    # make lines big
    canvas.setLineWidth(5)
    star(canvas, "Default: mitered join", "0: pointed", xcenter = 1*inch)
    canvas.setLineJoin(1)
    star(canvas, "Round join", "1: rounded")
    canvas.setLineJoin(2)
    star(canvas, "Bevelled join", "2: square", xcenter=4.5*inch)
"""

testcaps = """
def caps(canvas):
    from reportlab.lib.units import inch
    # make lines big
    canvas.setLineWidth(5)
    star(canvas, "Default", "no projection",xcenter = 1*inch,
         nvertices=4)
    canvas.setLineCap(1)
    star(canvas, "Round cap", "1: ends in half circle", nvertices=4)
    canvas.setLineCap(2)
    star(canvas, "Square cap", "2: projects out half a width", xcenter=4.5*inch,
       nvertices=4)
"""

testdashes = """
def dashes(canvas):
    from reportlab.lib.units import inch
    # make lines big
    canvas.setDash(6,3)
    star(canvas, "Simple dashes", "6 points on, 3 off", xcenter = 1*inch)
    canvas.setDash(1,2)
    star(canvas, "Dots", "One on, two off")
    canvas.setDash([1,1,3,3,1,4,4,1], 0)
    star(canvas, "Complex Pattern", "[1,1,3,3,1,4,4,1]", xcenter=4.5*inch)
"""

testcustomfont1 = """
def customfont1(canvas):
    # we know some glyphs are missing, suppress warnings
    import reportlab.rl_config
    reportlab.rl_config.warnOnMissingFontGlyphs = 0

    from . import rl_doc_utils
    from reportlab.pdfbase import pdfmetrics
    afmFile, pfbFile = rl_doc_utils.getJustFontPaths()
    justFace = pdfmetrics.EmbeddedType1Face(afmFile, pfbFile)
    faceName = 'DarkGardenMK' # pulled from AFM file
    pdfmetrics.registerTypeFace(justFace)
    justFont = pdfmetrics.Font('DarkGardenMK',
                               faceName,
                               'WinAnsiEncoding')
    pdfmetrics.registerFont(justFont)

    canvas.setFont('DarkGardenMK', 32)
    canvas.drawString(10, 150, 'This should be in')
    canvas.drawString(10, 100, 'DarkGardenMK')
"""

testttffont1 = """
def ttffont1(canvas):
    # we know some glyphs are missing, suppress warnings
    import reportlab.rl_config
    reportlab.rl_config.warnOnMissingFontGlyphs = 0
    from reportlab.pdfbase import pdfmetrics
    from reportlab.pdfbase.ttfonts import TTFont
    pdfmetrics.registerFont(TTFont('Vera', 'Vera.ttf'))
    from reportlab.pdfgen.canvas import Canvas

    canvas.setFont('Vera', 32)
    canvas.drawString(10, 150, "Some UTF-8 text encoded")
    canvas.drawString(10, 100, "in the Vera TT Font!")
"""

testcursormoves1 = """
def cursormoves1(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(inch, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 14)
    for line in lyrics:
        textobject.textLine(line)
    textobject.setFillGray(0.4)
    canvas.drawText(textobject)
"""

testcursormoves2 = """
def cursormoves2(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(2, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 14)
    for line in lyrics:
        textobject.textOut(line)
        textobject.moveCursor(14,14) # POSITIVE Y moves down!!!
    textobject.setFillColorRGB(0.4,0,1)
    canvas.drawText(textobject)
"""

testcharspace = """
def charspace(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(3, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 10)
    charspace = 0
    for line in lyrics:
        textobject.setCharSpace(charspace)
        textobject.textLine("%s: %s" %(charspace,line))
        charspace = charspace+0.5
    textobject.setFillGray(0.4)
    canvas.drawText(textobject)
"""

testwordspace = """
def wordspace(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(3, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 12)
    wordspace = 0
    for line in lyrics:
        textobject.setWordSpace(wordspace)
        textobject.textLine("%s: %s" %(wordspace,line))
        wordspace = wordspace+2.5
    textobject.setFillColorCMYK(0.4,0,0.4,0.2)
    canvas.drawText(textobject)
"""
testhorizontalscale = """
def horizontalscale(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(3, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 12)
    horizontalscale = 80 # 100 is default
    for line in lyrics:
        textobject.setHorizScale(horizontalscale)
        textobject.textLine("%s: %s" %(horizontalscale,line))
        horizontalscale = horizontalscale+10
    textobject.setFillColorCMYK(0.0,0.4,0.4,0.2)
    canvas.drawText(textobject)
"""
testleading = """
def leading(canvas):
    from reportlab.lib.units import inch
    textobject = canvas.beginText()
    textobject.setTextOrigin(3, 2.5*inch)
    textobject.setFont("Helvetica-Oblique", 14)
    leading = 8
    for line in lyrics:
        textobject.setLeading(leading)
        textobject.textLine("%s: %s" %(leading,line))
        leading = leading+2.5
    textobject.setFillColorCMYK(0.8,0,0,0.3)
    canvas.drawText(textobject)
"""

testhand = """
def hand(canvas, debug=1, fill=0):
    (startx, starty) = (0,0)
    curves = [
      ( 0, 2), ( 0, 4), ( 0, 8), # back of hand
      ( 5, 8), ( 7,10), ( 7,14),
      (10,14), (10,13), ( 7.5, 8), # thumb
      (13, 8), (14, 8), (17, 8),
      (19, 8), (19, 6), (17, 6),
      (15, 6), (13, 6), (11, 6), # index, pointing
      (12, 6), (13, 6), (14, 6),
      (16, 6), (16, 4), (14, 4),
      (13, 4), (12, 4), (11, 4), # middle
      (11.5, 4), (12, 4), (13, 4),
      (15, 4), (15, 2), (13, 2),
      (12.5, 2), (11.5, 2), (11, 2), # ring
      (11.5, 2), (12, 2), (12.5, 2),
      (14, 2), (14, 0), (12.5, 0),
      (10, 0), (8, 0), (6, 0), # pinky, then close
      ]
    from reportlab.lib.units import inch
    if debug: canvas.setLineWidth(6)
    u = inch*0.2
    p = canvas.beginPath()
    p.moveTo(startx, starty)
    ccopy = list(curves)
    while ccopy:
        [(x1,y1), (x2,y2), (x3,y3)] = ccopy[:3]
        del ccopy[:3]
        p.curveTo(x1*u,y1*u,x2*u,y2*u,x3*u,y3*u)
    p.close()
    canvas.drawPath(p, fill=fill)
    if debug:
        from reportlab.lib.colors import red, green
        (lastx, lasty) = (startx, starty)
        ccopy = list(curves)
        while ccopy:
            [(x1,y1), (x2,y2), (x3,y3)] = ccopy[:3]
            del ccopy[:3]
            canvas.setStrokeColor(red)
            canvas.line(lastx*u,lasty*u, x1*u,y1*u)
            canvas.setStrokeColor(green)
            canvas.line(x2*u,y2*u, x3*u,y3*u)
            (lastx,lasty) = (x3,y3)
"""

testhand2 = """
def hand2(canvas):
    canvas.translate(20,10)
    canvas.setLineWidth(3)
    canvas.setFillColorRGB(0.1, 0.3, 0.9)
    canvas.setStrokeGray(0.5)
    hand(canvas, debug=0, fill=1)
"""

testfonts = """
def fonts(canvas):
    from reportlab.lib.units import inch
    text = "Now is the time for all good men to..."
    x = 1.8*inch
    y = 2.7*inch
    for font in canvas.getAvailableFonts():
        canvas.setFont(font, 10)
        canvas.drawString(x,y,text)
        canvas.setFont("Helvetica", 10)
        canvas.drawRightString(x-10,y, font+":")
        y = y-13
"""

testarcs = """
def arcs(canvas):
    from reportlab.lib.units import inch
    canvas.setLineWidth(4)
    canvas.setStrokeColorRGB(0.8, 1, 0.6)
    # draw rectangles enclosing the arcs
    canvas.rect(inch, inch, 1.5*inch, inch)
    canvas.rect(3*inch, inch, inch, 1.5*inch)
    canvas.setStrokeColorRGB(0, 0.2, 0.4)
    canvas.setFillColorRGB(1, 0.6, 0.8)
    p = canvas.beginPath()
    p.moveTo(0.2*inch, 0.2*inch)
    p.arcTo(inch, inch, 2.5*inch,2*inch, startAng=-30, extent=135)
    p.arc(3*inch, inch, 4*inch, 2.5*inch, startAng=-45, extent=270)
    canvas.drawPath(p, fill=1, stroke=1)
"""
testvariousshapes = """
def variousshapes(canvas):
    from reportlab.lib.units import inch
    inch = int(inch)
    canvas.setStrokeGray(0.5)
    canvas.grid(range(0,int(11*inch/2),int(inch/2)), range(0,int(7*inch/2),int(inch/2)))
    canvas.setLineWidth(4)
    canvas.setStrokeColorRGB(0, 0.2, 0.7)
    canvas.setFillColorRGB(1, 0.6, 0.8)
    p = canvas.beginPath()
    p.rect(0.5*inch, 0.5*inch, 0.5*inch, 2*inch)
    p.circle(2.75*inch, 1.5*inch, 0.3*inch)
    p.ellipse(3.5*inch, 0.5*inch, 1.2*inch, 2*inch)
    canvas.drawPath(p, fill=1, stroke=1)
"""

testclosingfigures = """
def closingfigures(canvas):
    from reportlab.lib.units import inch
    h = inch/3.0; k = inch/2.0
    canvas.setStrokeColorRGB(0.2,0.3,0.5)
    canvas.setFillColorRGB(0.8,0.6,0.2)
    canvas.setLineWidth(4)
    p = canvas.beginPath()
    for i in (1,2,3,4):
        for j in (1,2):
            xc,yc = inch*i, inch*j
            p.moveTo(xc,yc)
            p.arcTo(xc-h, yc-k, xc+h, yc+k, startAng=0, extent=60*i)
            # close only the first one, not the second one
            if j==1:
                p.close()
    canvas.drawPath(p, fill=1, stroke=1)
"""

testforms = """
def forms(canvas):
    #first create a form...
    canvas.beginForm("SpumoniForm")
    #re-use some drawing functions from earlier
    spumoni(canvas)
    canvas.endForm()

    #then draw it
    canvas.doForm("SpumoniForm")
"""

def doctemplateillustration(canvas):
    from reportlab.lib.units import inch
    canvas.setFont("Helvetica", 10)
    canvas.drawString(inch/4.0, 2.75*inch, "DocTemplate")
    W = 4/3.0*inch
    H = 2*inch
    Wd = x = inch/4.0
    Hd =y = inch/2.0
    for name in ("two column", "chapter page", "title page"):
        canvas.setFillColorRGB(0.5,1.0,1.0)
        canvas.rect(x,y,W,H, fill=1)
        canvas.setFillColorRGB(0,0,0)
        canvas.drawString(x+inch/8, y+H-Wd, "PageTemplate")
        canvas.drawCentredString(x+W/2.0, y-Wd, name)
        x = x+W+Wd
    canvas.saveState()
    d = inch/16
    dW = (W-3*d)/2.0
    hD = H -2*d-Wd
    canvas.translate(Wd+d, Hd+d)
    for name in ("left Frame", "right Frame"):
        canvas.setFillColorRGB(1.0,0.5,1.0)
        canvas.rect(0,0, dW,hD, fill=1)
        canvas.setFillGray(0.7)
        dd= d/2.0
        ddH = (hD-6*dd)/5.0
        ddW = dW-2*dd
        yy = dd
        xx = dd
        for i in range(5):
            canvas.rect(xx,yy,ddW,ddH, fill=1, stroke=0)
            yy = yy+ddH+dd
        canvas.setFillColorRGB(0,0,0)
        canvas.saveState()
        canvas.rotate(90)
        canvas.drawString(d,-dW/2, name)
        canvas.restoreState()
        canvas.translate(dW+d,0)
    canvas.restoreState()
    canvas.setFillColorRGB(1.0, 0.5, 1.0)
    mx = Wd+W+Wd+d
    my = Hd+d
    mW = W-2*d
    mH = H-d-Hd
    canvas.rect(mx, my, mW, mH, fill=1)
    canvas.rect(Wd+2*(W+Wd)+d, Hd+3*d, W-2*d, H/2.0, fill=1)
    canvas.setFillGray(0.7)
    canvas.rect(Wd+2*(W+Wd)+d+dd, Hd+5*d, W-2*d-2*dd, H/2.0-2*d-dd, fill=1)
    xx = mx+dd
    yy = my+mH/5.0
    ddH = (mH-6*dd-mH/5.0)/3.0
    ddW = mW - 2*dd
    for i in range(3):
        canvas.setFillGray(0.7)
        canvas.rect(xx,yy,ddW,ddH, fill=1, stroke=1)
        canvas.setFillGray(0)
        canvas.drawString(xx+dd/2.0,yy+dd/2.0, "flowable %s" %(157-i))
        yy = yy+ddH+dd
    canvas.drawCentredString(3*Wd+2*W+W/2, Hd+H/2.0, "First Flowable")
    canvas.setFont("Times-BoldItalic", 8)
    canvas.setFillGray(0)
    canvas.drawCentredString(mx+mW/2.0, my+mH+3*dd, "Chapter 6: Lubricants")
    canvas.setFont("Times-BoldItalic", 10)
    canvas.drawCentredString(3*Wd+2*W+W/2, Hd+H-H/4, "College Life")

# D = dir()
TESTS=[(a,b) for a, b in globals().items() if a.startswith('test')]
for a,b in TESTS:
    if isinstance(b,str):
        b = b.strip()
        exec(b+'\n')

platypussetup = """
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch
PAGE_HEIGHT=defaultPageSize[1]; PAGE_WIDTH=defaultPageSize[0]
styles = getSampleStyleSheet()
"""
platypusfirstpage = """
Title = "Hello world"
pageinfo = "platypus example"
def myFirstPage(canvas, doc):
    canvas.saveState()
    canvas.setFont('Times-Bold',16)
    canvas.drawCentredString(PAGE_WIDTH/2.0, PAGE_HEIGHT-108, Title)
    canvas.setFont('Times-Roman',9)
    canvas.drawString(inch, 0.75 * inch, "First Page / %s" % pageinfo)
    canvas.restoreState()
"""
platypusnextpage = """
def myLaterPages(canvas, doc):
    canvas.saveState()
    canvas.setFont('Times-Roman',9)
    canvas.drawString(inch, 0.75 * inch, "Page %d %s" % (doc.page, pageinfo))
    canvas.restoreState()
"""
platypusgo = """
def go():
    doc = SimpleDocTemplate("phello.pdf")
    Story = [Spacer(1,2*inch)]
    style = styles["Normal"]
    for i in range(100):
        bogustext = ("This is Paragraph number %s.  " % i) *20
        p = Paragraph(bogustext, style)
        Story.append(p)
        Story.append(Spacer(1,0.2*inch))
    doc.build(Story, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
"""

if __name__=="__main__":
    # then do the platypus hello world
    for b in platypussetup, platypusfirstpage, platypusnextpage, platypusgo:
        b = b.strip()
        exec(b+'\n')
    go()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Line(30,70,170,70,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(30,70,30,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(58,70,58,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(86,70,86,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(114,70,114,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(142,70,142,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(170,70,170,65,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,44,65)
		v0.add(String(-8.33,-10,'One',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,72,65)
		v0.add(String(-9.165,-10,'Two',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,100,65)
		v0.add(String(-11.66,-10,'Three',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,128,65)
		v0.add(String(-9.445,-10,'Four',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (0,1,-1,0,156,65)
		v0.add(String(-17.78,-4,'Five',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
"""
Tests for RLG Image shapes.
"""
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation, rlSkipUnless
setOutDir(__name__)
import os
import unittest
from reportlab.graphics.shapes import Image, Drawing, Rect, Group, Line
from reportlab.graphics import renderPDF
from reportlab.lib.pagesizes import A4
from reportlab.lib.colors import toColor, CMYKColor, rgb2cmyk

IMAGENAME = 'pythonpowered.gif'
GSIMAGE = 'pythonpowered-gs.gif'
GAIMAGE = 'gray-alpha.png'

try:
    import rlPyCairo
except ImportError:
    rlPyCairo = None

class ImageTestCase(unittest.TestCase):
    "Test RLG Image shape."

    @classmethod
    def setUpClass(cls):
        cls.IMAGES = []
    
    @classmethod
    def tearDownClass(cls):
        if not cls.IMAGES: return
        d = Drawing(A4[0], A4[1])
        for img in cls.IMAGES:
            d.add(img)
        outPath = outputfile("test_graphics_images.pdf")
        renderPDF.drawToFile(d, outPath) #, '')
        assert os.path.exists(outPath)

        try:
            import _rl_renderPM
        except ImportError:
            _rl_renderPM = None

        from reportlab.rl_config import renderPMBackend
        if rlPyCairo:
            d.save(formats=['png', 'gif', 'ps','svg'],outDir=os.path.dirname(outPath), fnRoot='test_graphics_images', _renderPM_backend='rlPyCairo')
        if _rl_renderPM:
            d.save(formats=['png', 'gif'],outDir=os.path.dirname(outPath), fnRoot='test_graphics_images-libart', _renderPM_backend='_renderPM')


    def test0(self):
        "Test convert a bitmap file as Image shape into a tmp. PDF file."

        d = Drawing(110, 44)
        inPath = IMAGENAME
        img = Image(0, 0, 110, 44, inPath)
        d.add(img)
        self.IMAGES.append(img)

    def test1(self):
        "Test Image shape, adding it to a PDF page."

        inPath = IMAGENAME
        img = Image(0, 0, 110, 44, inPath)
        self.IMAGES.append(img)


    def test2(self):
        "Test scaled Image shape adding it to a PDF page."

        inPath = IMAGENAME
        img = Image(0, 0, 110, 44, inPath)
        d = Drawing(110, 44)
        d.add(img)
        d.translate(120, 0)
        d.scale(2, 2)
        self.IMAGES.append(d)

    def test3(self):
        "Test rotated Image shape adding it to a PDF page."

        inPath = IMAGENAME
        img = Image(0, 0, 110, 44, inPath)
        d = Drawing(110, 44)
        d.add(img)
        d.translate(420, 0)
        d.scale(2, 2)
        d.rotate(45)
        self.IMAGES.append(d)

    def test4(self):
        "Test convert a greyscale bitmap file as Image shape into a tmp. PDF file."

        d = Drawing(110, 44)
        img = Image(0, 0, 110, 44, GSIMAGE)
        d.add(img)
        d.translate(0,2*72)
        self.IMAGES.append(d)

    def test5(self):
        "Test convert a greyscale +alpha bitmap file as Image shape into a tmp. PDF file."

        d = Drawing(48, 48)
        img = Image(0, 0, 48, 48, GAIMAGE)
        d.add(img)
        d.translate(72,4*72)
        self.IMAGES.append(d)

    def test6(self):
        d = Drawing(200, 100)
        d.add(Rect(1,1,d.width-2,d.height-2,strokeWidth=2,strokeColor=toColor('red'),fillColor=toColor('lightblue')),name='bg0')
        def addImage(x,y,w,h):
            img = Image(x, y, w, h, IMAGENAME)
            d.add(Rect(img.x-1,img.y-1,img.width+2,img.height+2,strokeWidth=2,strokeColor=toColor('green'),fillColor=toColor('black')),name='bg1')
            d.add(img)
        addImage(40,60,55,22)
        addImage(10,10,110,44)
        d.translate(72,6*72)
        self.IMAGES.append(d)

    @rlSkipUnless(rlPyCairo,'cannot import rlPyCairo')
    def test7(self):
        rw = 10
        rg = 10
        xgap = 5
        h = w = 3*rw+4*rg
        d = Drawing(2*h + xgap,h)
        C = toColor('red'),toColor('green'),toColor('blue')
        g = Group()

        def colconv(j,c):
            if j==0: return c
            if j==1: return CMYKColor(*rgb2cmyk(c.red,c.green,c.blue),alpha=c.alpha)

        from reportlab.graphics.widgets.grids import ShadedRect
        for i in range(3):
            c = C[i]
            sc = c.clone(alpha=0.1)
            ec = c.clone(alpha=1.0)
            for j in range(2):
                x = j*(w+xgap)
                d.add(ShadedRect(x=x,y=rg+(rg+rw)*i,width=w,height=rw,orientation='vertical',numShades=w//10,
                        fillColorStart=colconv(j,sc),fillColorEnd=colconv(j,ec),
                        strokeColor=None,strokeWidth=0))

                g.add(ShadedRect(x=x+rg+(rg+rw)*i,y=0,width=rw,height=h,orientation='horizontal',numShades=w//10,
                        fillColorStart=colconv(j,sc),fillColorEnd=colconv(j,ec),
                        strokeColor=None,strokeWidth=0))
        d.add(g)

        d._renderPM_backendFmt = 'ARGB32'
        d._renderPM_bg = toColor('white').clone(alpha=0)    #transparent white
        d.save(formats=['png'],
            fnRoot=outputfile(os.path.join("charts-out","test_graphics_images_alpha_test.png")), outDir=None)

    @rlSkipUnless(rlPyCairo,'cannot import rlPyCairo')
    def test8(self):
        rw = 10
        rg = 10
        sw = 2
        xgap = 5
        h = w = 3*rw+4*rg
        d = Drawing(2*h + xgap,h)
        C = toColor('red'),toColor('green')
        g = Group()

        def colconv(j,c):
            if j==0: return c
            if j==1: return CMYKColor(*rgb2cmyk(c.red,c.green,c.blue),alpha=c.alpha)

        for i in range(3):
            ch = C[0].clone(alpha=0.1 + 0.45*i)
            cv = C[1].clone(alpha=0.1 + 0.45*i)
            for j in range(2):
                x = j*(w+xgap)
                y = rg+(rg+rw)*i + rw/2
                d.add(Line(x,y,x+w,y, strokeColor=ch, strokeWidth=sw))
                x += rg+(rg+rw)*i+ rw/2
                g.add(Line(x,0,x, h, strokeColor=cv, strokeWidth=sw))
        d.add(g)

        d._renderPM_backendFmt = 'ARGB32'
        d._renderPM_bg = toColor('white').clone(alpha=0)    #transparent white
        d.save(formats=['png'],
            fnRoot=outputfile(os.path.join("charts-out","test_graphics_images_alpha_test8.png")), outDir=None)

def makeSuite():
    return makeSuiteForClasses(ImageTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Circle, Rect, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Circle(100,50,50,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Circle(100,50,48,fillColor=Color(1,.270588,0,1),fillOpacity=None,strokeColor=None,strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(60,40,80,20,rx=0,ry=0,fillColor=Color(.972549,.972549,1,1),fillOpacity=None,strokeColor=Color(.972549,.972549,1,1),strokeWidth=0,strokeLineCap=1,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(100,-12,'NoEntry',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=100,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(0,0,5,100,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=Color(1,.752941,.796078,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(5,0,5,100,rx=0,ry=0,fillColor=Color(.947368,.713313,.75418,1),fillOpacity=None,strokeColor=Color(.947368,.713313,.75418,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(10,0,5,100,rx=0,ry=0,fillColor=Color(.894737,.673684,.712281,1),fillOpacity=None,strokeColor=Color(.894737,.673684,.712281,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(15,0,5,100,rx=0,ry=0,fillColor=Color(.842105,.634056,.670382,1),fillOpacity=None,strokeColor=Color(.842105,.634056,.670382,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(20,0,5,100,rx=0,ry=0,fillColor=Color(.789474,.594427,.628483,1),fillOpacity=None,strokeColor=Color(.789474,.594427,.628483,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(25,0,5,100,rx=0,ry=0,fillColor=Color(.736842,.554799,.586584,1),fillOpacity=None,strokeColor=Color(.736842,.554799,.586584,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(30,0,5,100,rx=0,ry=0,fillColor=Color(.684211,.51517,.544685,1),fillOpacity=None,strokeColor=Color(.684211,.51517,.544685,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(35,0,5,100,rx=0,ry=0,fillColor=Color(.631579,.475542,.502786,1),fillOpacity=None,strokeColor=Color(.631579,.475542,.502786,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(40,0,5,100,rx=0,ry=0,fillColor=Color(.578947,.435913,.460888,1),fillOpacity=None,strokeColor=Color(.578947,.435913,.460888,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(45,0,5,100,rx=0,ry=0,fillColor=Color(.526316,.396285,.418989,1),fillOpacity=None,strokeColor=Color(.526316,.396285,.418989,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,0,5,100,rx=0,ry=0,fillColor=Color(.473684,.356656,.37709,1),fillOpacity=None,strokeColor=Color(.473684,.356656,.37709,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(55,0,5,100,rx=0,ry=0,fillColor=Color(.421053,.317028,.335191,1),fillOpacity=None,strokeColor=Color(.421053,.317028,.335191,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(60,0,5,100,rx=0,ry=0,fillColor=Color(.368421,.277399,.293292,1),fillOpacity=None,strokeColor=Color(.368421,.277399,.293292,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(65,0,5,100,rx=0,ry=0,fillColor=Color(.315789,.237771,.251393,1),fillOpacity=None,strokeColor=Color(.315789,.237771,.251393,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(70,0,5,100,rx=0,ry=0,fillColor=Color(.263158,.198142,.209494,1),fillOpacity=None,strokeColor=Color(.263158,.198142,.209494,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(75,0,5,100,rx=0,ry=0,fillColor=Color(.210526,.158514,.167595,1),fillOpacity=None,strokeColor=Color(.210526,.158514,.167595,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(80,0,5,100,rx=0,ry=0,fillColor=Color(.157895,.118885,.125697,1),fillOpacity=None,strokeColor=Color(.157895,.118885,.125697,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(85,0,5,100,rx=0,ry=0,fillColor=Color(.105263,.079257,.083798,1),fillOpacity=None,strokeColor=Color(.105263,.079257,.083798,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(90,0,5,100,rx=0,ry=0,fillColor=Color(.052632,.039628,.041899,1),fillOpacity=None,strokeColor=Color(.052632,.039628,.041899,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,0,5,100,rx=0,ry=0,fillColor=Color(0,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(0,0,100,100,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Copyright ReportLab Europe Ltd. 2000-2025
#see LICENSE for license details
__version__='4.4.5'
import os, sys, glob, shutil, re, sysconfig, traceback, io, subprocess
from urllib.parse import quote as urlquote
platform = sys.platform
pjoin = os.path.join
abspath = os.path.abspath
isfile = os.path.isfile
isdir = os.path.isdir
dirname = os.path.dirname
basename = os.path.basename
splitext = os.path.splitext
addrSize = 64 if sys.maxsize > 2**32 else 32
sysconfig_platform = sysconfig.get_platform()

INFOLINES=[]
def infoline(t,
        pfx='#####',
        add=True,
        ):
    bn = splitext(basename(sys.argv[0]))[0]
    ver = '.'.join(map(str,sys.version_info[:3]))
    s = '%s %s-python-%s-%s: %s' % (pfx, bn, ver, sysconfig_platform, t)
    print(s)
    if add: INFOLINES.append(s)

def showTraceback(s):
    buf = io.StringIO()
    print(s,file=buf)
    if verbose>2:
        traceback.print_exc(file=buf)
    for l in buf.getvalue().split('\n'):
        infoline(l,pfx='!!!!!',add=False)

def spCall(cmd,*args,**kwds):
    r = subprocess.call(
            cmd,
            stderr = subprocess.STDOUT,
            stdout = subprocess.DEVNULL if kwds.pop('dropOutput',False) else None,
            timeout = kwds.pop('timeout',3600),
            )
    if verbose>=3:
        infoline('%r --> %s' % (' '.join(cmd),r), pfx='!!!!!' if r else '#####', add=False)
    return r

def specialOption(n,ceq=False):
    v = 0
    while n in sys.argv:
        v += 1
        sys.argv.remove(n)
    if ceq:
        n += '='
        V = [_ for _ in sys.argv if _.startswith(n)]
        for _ in V: sys.argv.remove(_)
        if V:
            n = len(n)
            v = V[-1][n:]
    return v

dlt1 = not specialOption('--no-download-t1-files')
verbose = specialOption('--verbose',ceq=True)
nullDivert = not verbose

pkgDir=dirname(__file__)
if not pkgDir:
    pkgDir=os.getcwd()
elif not os.path.isabs(pkgDir):
    pkgDir=abspath(pkgDir)
try:
    os.chdir(pkgDir)
except:
    showTraceback('warning could not change directory to %r' % pkgDir)

def get_version():
    #determine Version

    #first try source
    FN = pjoin(pkgDir,'src','reportlab','__init__')
    try:
        for l in open(pjoin(FN+'.py'),'r').readlines():
            if l.startswith('Version'):
                D = {}
                exec(l.strip(),D)
                return D['Version']
    except:
        pass

    #don't have source, try import
    import imp
    for desc in ('.pyc', 'rb', 2), ('.pyo', 'rb', 2):
        try:
            fn = FN+desc[0]
            f = open(fn,desc[1])
            m = imp.load_module('reportlab',f,fn,desc)
            return m.Version
        except:
            pass
    raise ValueError('Cannot determine ReportLab Version')

#this code from /FBot's PIL setup.py
def aDir(P, d, x=None):
    if d and isdir(d) and d not in P:
        if x is None:
            P.append(d)
        else:
            P.insert(x, d)

# protection against loops needed. reported by
# Michał Górny &lt; mgorny at gentoo dot org &gt;
# see https://stackoverflow.com/questions/36977259
def findFile(root, wanted, followlinks=True):
    visited = set()
    for p, D, F in os.walk(root,followlinks=followlinks):
        #scan directories to check for prior visits
        #use dev/inode to make unique key
        SD = [].append
        for d in D:
            dk = os.stat(pjoin(p,d))
            dk = dk.st_dev, dk.st_ino
            if dk not in visited:
                visited.add(dk)
                SD(d)
        D[:] = SD.__self__  #set the dirs to be scanned
        for fn in F:
            if fn==wanted:  
                return abspath(pjoin(p,fn))

def listFiles(root,followlinks=True,strJoin=None):
    visited = set()
    R = [].append
    for p, D, F in os.walk(root,followlinks=followlinks):
        #scan directories to check for prior visits
        #use dev/inode to make unique key
        SD = [].append
        for d in D:
            dk = os.stat(pjoin(p,d))
            dk = dk.st_dev, dk.st_ino
            if dk not in visited:
                visited.add(dk)
                SD(d)
        D[:] = SD.__self__  #set the dirs to be scanned
        for fn in F:
            R(abspath(pjoin(p,fn)))
    R = R.__self__
    return strJoin.join(R) if strJoin else R

reportlab_files= [
        'fonts/00readme.txt',
        'fonts/bitstream-vera-license.txt',
        'fonts/hb-test.ttf',
        'fonts/DarkGarden-changelog.txt',
        'fonts/DarkGarden-copying-gpl.txt',
        'fonts/DarkGarden-copying.txt',
        'fonts/DarkGarden-readme.txt',
        'fonts/DarkGarden.sfd',
        'fonts/DarkGardenMK.afm',
        'fonts/DarkGardenMK.pfb',
        'fonts/Vera.ttf',
        'fonts/VeraBd.ttf',
        'fonts/VeraBI.ttf',
        'fonts/VeraIt.ttf',
        'fonts/_abi____.pfb',
        'fonts/_ab_____.pfb',
        'fonts/_ai_____.pfb',
        'fonts/_a______.pfb',
        'fonts/cobo____.pfb',
        'fonts/cob_____.pfb',
        'fonts/com_____.pfb',
        'fonts/coo_____.pfb',
        'fonts/_ebi____.pfb',
        'fonts/_eb_____.pfb',
        'fonts/_ei_____.pfb',
        'fonts/_er_____.pfb',
        'fonts/sy______.pfb',
        'fonts/zd______.pfb',
        'fonts/zx______.pfb',
        'fonts/zy______.pfb',
        'fonts/callig15.pfb',
        'fonts/callig15.afm',
        'reportlab/graphics/barcode/README'
        'reportlab/graphics/barcode/TODO'
        'license.txt',
        ]

def url2data(url,returnRaw=False):
    import urllib.request as ureq
    remotehandle = ureq.urlopen(url)
    try:
        raw = remotehandle.read()
        return raw if returnRaw else io.BytesIO(raw)
    finally:
        remotehandle.close()

def get_fonts(PACKAGE_DIR, reportlab_files):
    import zipfile
    rl_dir = PACKAGE_DIR['reportlab']
    if not [x for x in reportlab_files if not isfile(pjoin(rl_dir,x))]:
        xitmsg = "Standard T1 font curves already downloaded"
    elif not dlt1:
        xitmsg = "not downloading T1 font curve files"
    else:
        try:
            infoline("Downloading standard T1 font curves")
            zipdata = url2data("https://www.reportlab.com/ftp/pfbfer-20180109.zip")
            archive = zipfile.ZipFile(zipdata)
            dst = pjoin(rl_dir, 'fonts')

            for name in archive.namelist():
                if not name.endswith('/'):
                    with open(pjoin(dst, name), 'wb') as outfile:
                        outfile.write(archive.read(name))
            xitmsg = "Finished download of standard T1 font curves"
        except:
            xitmsg = "Failed to download standard T1 font curves"
    infoline(xitmsg)

def get_glyphlist_module(PACKAGE_DIR):
    try:
        lfn = pjoin("pdfbase","_glyphlist.py")
        fn = pjoin(PACKAGE_DIR['reportlab'],lfn)
        if isfile(fn):
            xitmsg = "The _glyphlist module already exists"
        else:
            text = url2data("https://raw.githubusercontent.com/adobe-type-tools/agl-aglfn/master/glyphlist.txt",True)
            comments = ['#see https://github.com/adobe-type-tools/agl-aglfn\n'].append
            G2U = [].append
            G2Us = [].append
            if not isinstance(text,str):
                text = text.decode('latin1')
            for line in text.split('\n'):
                line = line.strip()
                if not line: continue
                if line.startswith('#'):
                    comments(line+'\n')
                else:
                    gu = line.split(';')
                    if len(gu)==2:
                        v = gu[1].split()
                        if len(v)==1:
                            G2U('\t%r: 0x%s,\n' % (gu[0],gu[1]))
                        else:
                            G2Us('\t%r: (%s),\n' % (gu[0],','.join('0x%s'%u for u in v)))
                    else:
                        infoline('bad glyphlist line %r' % line, '!!!!!')
            with open(fn,'w') as f:
                f.write(''.join(comments.__self__))
                f.write('_glyphname2unicode = {\n')
                f.write(''.join(G2U.__self__))
                f.write('\t}\n')
                f.write('_glyphname2unicodes = {\n')
                f.write(''.join(G2Us.__self__))
                f.write('\t}\n')
            xitmsg = "Finished creation of _glyphlist.py"
    except:
        xitmsg = "Failed to download glyphlist.txt"
    infoline(xitmsg)

def main():
    if 'test' in sys.argv \
        or 'tests' in sys.argv \
        or 'tests-postinstall' in sys.argv \
        or 'tests-preinstall' in sys.argv:
        failfast = specialOption('--failfast')
        verboseTests = specialOption('--verbose-tests')
        excludes = [_ for _ in sys.argv if _.startswith('--exclude=')]
        for _ in excludes:
            sys.argv.remove(_)
        if len(sys.argv)!=2:
            raise ValueError('tests commands may only be used alone sys.argv[1:]=%s' % repr(sys.argv[1:]))
        cmd = sys.argv[-1]
        PYTHONPATH = [pkgDir] if cmd!='test' else []
        if cmd=='tests-preinstall':
            PYTHONPATH.insert(0,pjoin(pkgDir,'src'))
        if PYTHONPATH: os.environ['PYTHONPATH']=os.pathsep.join(PYTHONPATH)
        os.chdir(pjoin(pkgDir,'tests'))
        cli = [sys.executable, 'runAll.py']+excludes
        if cmd=='tests-postinstall':
            cli.append('--post-install')
        if verboseTests:
            cli.append('--verbosity=2')
        if failfast:
            cli.append('--failfast')
        r = spCall(cli)
        sys.exit(('!!!!! runAll.py --> %s should exit with error !!!!!' % r) if r else r)

    #copy some special case files into place so package_data will treat them properly
    PACKAGE_DIR = {'':'src','reportlab': pjoin('src','reportlab')}
    get_fonts(PACKAGE_DIR, reportlab_files)
    get_glyphlist_module(PACKAGE_DIR)

    from setuptools import setup
    setup(
        name="reportlab",
        version=get_version(),
        license="BSD license (see license.txt for details), Copyright (c) 2000-2025, ReportLab Inc.",
        description="The Reportlab Toolkit",
        long_description="""The ReportLab Toolkit. An Open Source Python library for generating PDFs and graphics.""",

        author="Andy Robinson, Robin Becker, the ReportLab team and the community",
        author_email="reportlab-users@lists2.reportlab.com",
        url="https://www.reportlab.com/",
        packages=[
                'reportlab',
                'reportlab.graphics.charts',
                'reportlab.graphics.samples',
                'reportlab.graphics.widgets',
                'reportlab.graphics.barcode',
                'reportlab.graphics',
                'reportlab.lib',
                'reportlab.pdfbase',
                'reportlab.pdfgen',
                'reportlab.platypus',
                ],
        package_dir = PACKAGE_DIR,
        package_data = {'reportlab': reportlab_files},
        classifiers = [
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Topic :: Printing',
            'Topic :: Text Processing :: Markup',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3.11',
            'Programming Language :: Python :: 3.12',
            'Programming Language :: Python :: 3.13',
            'Programming Language :: Python :: 3.14',
            ],
        
        # this probably only works for setuptools, but distutils seems to ignore it
        install_requires=['pillow>=9.0.0','charset-normalizer'],
        # moved to pyproject.toml
        python_requires='>=3.9,<4',
        extras_require={
            'accel': ['rl_accel>=0.9.0,<1.1'],
            'renderpm': ['rl_renderPM>=4.0.3,<4.1'],
            'pycairo': ['rlPyCairo>=0.2.0,<1', 'freetype-py>=2.3.0,<2.4'],
            'bidi': ['rlbidi'],
            'shaping': ['uharfbuzz'],
            },
        )
    print()
    print('########## SUMMARY INFO #########')
    print('\n'.join(INFOLINES))

if __name__=='__main__':
    main()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, Line, Circle, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=76.85768503937007,height=78.85568503937007,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(30,5,42.51969,42.51969,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(30,19.17323,72.51969,19.17323,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(30,33.34646,72.51969,33.34646,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(44.17323,5,44.17323,47.51969,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(58.34646,5,58.34646,47.51969,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Circle(44.17323,19.17323,5.669291,fillColor=Color(.909804,.878431,.466667,1),fillOpacity=None,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(25,-2113.567,'Large',textAnchor='end',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100),strokeColor=Color(.098039,.301961,.529412,1)))
		self.add(String(25,-2127.74,'Medium',textAnchor='end',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100),strokeColor=Color(.098039,.301961,.529412,1)))
		self.add(String(25,-2141.913,'Small',textAnchor='end',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100),strokeColor=Color(.098039,.301961,.529412,1)))
		v0=self._nn(Group())
		v0.transform = (0,1,-1,0,2191.087,52.51969)
		v0.add(String(0,1.2,'Value',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100)))
		v0=self._nn(Group())
		v0.transform = (0,1,-1,0,2205.26,52.51969)
		v0.add(String(0,1.2,'Blend',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100)))
		v0=self._nn(Group())
		v0.transform = (0,1,-1,0,2219.433,52.51969)
		v0.add(String(0,1.2,'Growth',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=PCMYKColor(0,0,0,100,alpha=100)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Circle, Rect, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Circle(100,90,5,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (.707107,.707107,-0.707107,.707107,100,70)
		v0.add(Rect(-43.89,-36,43.89,36,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,.501961,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(-43.89,-10,'Another',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0.add(String(-43.89,-22,'Multi-Line',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0.add(String(-43.89,-34,'String',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Wedge, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Wedge(100,50,50,72.85714,90,yradius=40,annular=False,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None,radius1=20,yradius1=16))
		self.add(Wedge(100,50,50,38.57143,72.85714,yradius=40,annular=False,fillColor=Color(.541176,.168627,.886275,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None,radius1=20,yradius1=16))
		self.add(Wedge(100,50,50,-12.85714,38.57143,yradius=40,annular=False,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None,radius1=20,yradius1=16))
		self.add(Wedge(106.8017,42.66948,50,-81.42857,-12.85714,yradius=40,annular=False,fillColor=Color(0,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=[2,2],strokeOpacity=None,radius1=20,yradius1=16))
		self.add(Wedge(100,50,50,-167.1429,-81.42857,yradius=40,annular=False,fillColor=Color(.498039,1,.831373,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None,radius1=20,yradius1=16))
		self.add(Wedge(100,50,50,-270,-167.1429,yradius=40,annular=False,fillColor=Color(.372549,.619608,.627451,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None,radius1=20,yradius1=16))
		self.add(String(108.9425,97.46388,'a',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(133.7992,89.65946,'b',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(158.4957,60.681,'c',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(159.5151,-1.313631,'d',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(1,0,0,1)))
		self.add(String(66.2008,10.34054,'e',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(53.09011,79.92751,'f',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)
